/****************************************************************************
** Meta object code from reading C++ file 'qstate.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../qtscxml/src/statemachine/qstate.h"
#include <QtCore/qmetatype.h>
#include <QtCore/QProperty>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qstate.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN6QStateE_t {};
} // unnamed namespace

template <> constexpr inline auto QState::qt_create_metaobjectdata<qt_meta_tag_ZN6QStateE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QState",
        "finished",
        "",
        "propertiesAssigned",
        "childModeChanged",
        "initialStateChanged",
        "errorStateChanged",
        "initialState",
        "QAbstractState*",
        "errorState",
        "childMode",
        "ChildMode",
        "ExclusiveStates",
        "ParallelStates",
        "RestorePolicy",
        "DontRestoreProperties",
        "RestoreProperties"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'finished'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'propertiesAssigned'
        QtMocHelpers::SignalData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'childModeChanged'
        QtMocHelpers::SignalData<void()>(4, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'initialStateChanged'
        QtMocHelpers::SignalData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'errorStateChanged'
        QtMocHelpers::SignalData<void()>(6, 2, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'initialState'
        QtMocHelpers::PropertyData<QAbstractState*>(7, 0x80000000 | 8, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet | QMC::Bindable, 3),
        // property 'errorState'
        QtMocHelpers::PropertyData<QAbstractState*>(9, 0x80000000 | 8, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet | QMC::Bindable, 4),
        // property 'childMode'
        QtMocHelpers::PropertyData<enum ChildMode>(10, 0x80000000 | 11, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet | QMC::Bindable, 2),
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'ChildMode'
        QtMocHelpers::EnumData<enum ChildMode>(11, 11, QMC::EnumFlags{}).add({
            {   12, ChildMode::ExclusiveStates },
            {   13, ChildMode::ParallelStates },
        }),
        // enum 'RestorePolicy'
        QtMocHelpers::EnumData<enum RestorePolicy>(14, 14, QMC::EnumFlags{}).add({
            {   15, RestorePolicy::DontRestoreProperties },
            {   16, RestorePolicy::RestoreProperties },
        }),
    };
    return QtMocHelpers::metaObjectData<QState, qt_meta_tag_ZN6QStateE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject QState::staticMetaObject = { {
    QMetaObject::SuperData::link<QAbstractState::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6QStateE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6QStateE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN6QStateE_t>.metaTypes,
    nullptr
} };

void QState::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QState *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->finished(QPrivateSignal()); break;
        case 1: _t->propertiesAssigned(QPrivateSignal()); break;
        case 2: _t->childModeChanged(QPrivateSignal()); break;
        case 3: _t->initialStateChanged(QPrivateSignal()); break;
        case 4: _t->errorStateChanged(QPrivateSignal()); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (QState::*)(QPrivateSignal)>(_a, &QState::finished, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (QState::*)(QPrivateSignal)>(_a, &QState::propertiesAssigned, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (QState::*)(QPrivateSignal)>(_a, &QState::childModeChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (QState::*)(QPrivateSignal)>(_a, &QState::initialStateChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (QState::*)(QPrivateSignal)>(_a, &QState::errorStateChanged, 4))
            return;
    }
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 1:
        case 0:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QAbstractState* >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QAbstractState**>(_v) = _t->initialState(); break;
        case 1: *reinterpret_cast<QAbstractState**>(_v) = _t->errorState(); break;
        case 2: *reinterpret_cast<enum ChildMode*>(_v) = _t->childMode(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setInitialState(*reinterpret_cast<QAbstractState**>(_v)); break;
        case 1: _t->setErrorState(*reinterpret_cast<QAbstractState**>(_v)); break;
        case 2: _t->setChildMode(*reinterpret_cast<enum ChildMode*>(_v)); break;
        default: break;
        }
    }
    if (_c == QMetaObject::BindableProperty) {
        switch (_id) {
        case 0: *static_cast<QUntypedBindable *>(_a[0]) = _t->bindableInitialState(); break;
        case 1: *static_cast<QUntypedBindable *>(_a[0]) = _t->bindableErrorState(); break;
        case 2: *static_cast<QUntypedBindable *>(_a[0]) = _t->bindableChildMode(); break;
        default: break;
        }
    }
}

const QMetaObject *QState::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QState::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6QStateE_t>.strings))
        return static_cast<void*>(this);
    return QAbstractState::qt_metacast(_clname);
}

int QState::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QAbstractState::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 5)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 5)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 5;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    }
    return _id;
}

// SIGNAL 0
void QState::finished(QPrivateSignal _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void QState::propertiesAssigned(QPrivateSignal _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void QState::childModeChanged(QPrivateSignal _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void QState::initialStateChanged(QPrivateSignal _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void QState::errorStateChanged(QPrivateSignal _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}
QT_WARNING_POP
