/****************************************************************************
** Meta object code from reading C++ file 'qhistorystate.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../qtscxml/src/statemachine/qhistorystate.h"
#include <QtCore/qmetatype.h>
#include <QtCore/QProperty>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qhistorystate.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN13QHistoryStateE_t {};
} // unnamed namespace

template <> constexpr inline auto QHistoryState::qt_create_metaobjectdata<qt_meta_tag_ZN13QHistoryStateE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QHistoryState",
        "defaultTransitionChanged",
        "",
        "defaultStateChanged",
        "historyTypeChanged",
        "defaultState",
        "QAbstractState*",
        "defaultTransition",
        "QAbstractTransition*",
        "historyType",
        "HistoryType",
        "ShallowHistory",
        "DeepHistory"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'defaultTransitionChanged'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'defaultStateChanged'
        QtMocHelpers::SignalData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'historyTypeChanged'
        QtMocHelpers::SignalData<void()>(4, 2, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'defaultState'
        QtMocHelpers::PropertyData<QAbstractState*>(5, 0x80000000 | 6, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 1),
        // property 'defaultTransition'
        QtMocHelpers::PropertyData<QAbstractTransition*>(7, 0x80000000 | 8, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet | QMC::Bindable, 0),
        // property 'historyType'
        QtMocHelpers::PropertyData<enum HistoryType>(9, 0x80000000 | 10, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet | QMC::Bindable, 2),
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'HistoryType'
        QtMocHelpers::EnumData<enum HistoryType>(10, 10, QMC::EnumFlags{}).add({
            {   11, HistoryType::ShallowHistory },
            {   12, HistoryType::DeepHistory },
        }),
    };
    return QtMocHelpers::metaObjectData<QHistoryState, qt_meta_tag_ZN13QHistoryStateE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject QHistoryState::staticMetaObject = { {
    QMetaObject::SuperData::link<QAbstractState::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13QHistoryStateE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13QHistoryStateE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN13QHistoryStateE_t>.metaTypes,
    nullptr
} };

void QHistoryState::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QHistoryState *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->defaultTransitionChanged(QPrivateSignal()); break;
        case 1: _t->defaultStateChanged(QPrivateSignal()); break;
        case 2: _t->historyTypeChanged(QPrivateSignal()); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (QHistoryState::*)(QPrivateSignal)>(_a, &QHistoryState::defaultTransitionChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (QHistoryState::*)(QPrivateSignal)>(_a, &QHistoryState::defaultStateChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (QHistoryState::*)(QPrivateSignal)>(_a, &QHistoryState::historyTypeChanged, 2))
            return;
    }
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 0:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QAbstractState* >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QAbstractState**>(_v) = _t->defaultState(); break;
        case 1: *reinterpret_cast<QAbstractTransition**>(_v) = _t->defaultTransition(); break;
        case 2: *reinterpret_cast<enum HistoryType*>(_v) = _t->historyType(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setDefaultState(*reinterpret_cast<QAbstractState**>(_v)); break;
        case 1: _t->setDefaultTransition(*reinterpret_cast<QAbstractTransition**>(_v)); break;
        case 2: _t->setHistoryType(*reinterpret_cast<enum HistoryType*>(_v)); break;
        default: break;
        }
    }
    if (_c == QMetaObject::BindableProperty) {
        switch (_id) {
        case 1: *static_cast<QUntypedBindable *>(_a[0]) = _t->bindableDefaultTransition(); break;
        case 2: *static_cast<QUntypedBindable *>(_a[0]) = _t->bindableHistoryType(); break;
        default: break;
        }
    }
}

const QMetaObject *QHistoryState::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QHistoryState::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13QHistoryStateE_t>.strings))
        return static_cast<void*>(this);
    return QAbstractState::qt_metacast(_clname);
}

int QHistoryState::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QAbstractState::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 3)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 3)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 3;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    }
    return _id;
}

// SIGNAL 0
void QHistoryState::defaultTransitionChanged(QPrivateSignal _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void QHistoryState::defaultStateChanged(QPrivateSignal _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void QHistoryState::historyTypeChanged(QPrivateSignal _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}
QT_WARNING_POP
