/****************************************************************************
** Meta object code from reading C++ file 'qscxmlstatemachine.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../qtscxml/src/scxml/qscxmlstatemachine.h"
#include "qscxmlstatemachine.h"
#include "qscxmltabledata.h"
#include <QtCore/qmetatype.h>
#include <QtCore/QList>
#include <QtCore/QProperty>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qscxmlstatemachine.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN18QScxmlStateMachineE_t {};
} // unnamed namespace

template <> constexpr inline auto QScxmlStateMachine::qt_create_metaobjectdata<qt_meta_tag_ZN18QScxmlStateMachineE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QScxmlStateMachine",
        "runningChanged",
        "",
        "running",
        "invokedServicesChanged",
        "QList<QScxmlInvokableService*>",
        "invokedServices",
        "log",
        "label",
        "msg",
        "reachedStableState",
        "finished",
        "dataModelChanged",
        "QScxmlDataModel*",
        "model",
        "initialValuesChanged",
        "QVariantMap",
        "initialValues",
        "initializedChanged",
        "initialized",
        "loaderChanged",
        "QScxmlCompiler::Loader*",
        "loader",
        "tableDataChanged",
        "QScxmlTableData*",
        "tableData",
        "start",
        "stop",
        "init",
        "stateNames",
        "compress",
        "activeStateNames",
        "isActive",
        "scxmlStateName",
        "submitEvent",
        "QScxmlEvent*",
        "event",
        "eventName",
        "QVariant",
        "data",
        "cancelDelayedEvent",
        "sendId",
        "isDispatchableTarget",
        "target",
        "dataModel",
        "sessionId",
        "name",
        "invoked",
        "parseErrors",
        "QList<QScxmlError>"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'runningChanged'
        QtMocHelpers::SignalData<void(bool)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'invokedServicesChanged'
        QtMocHelpers::SignalData<void(const QList<QScxmlInvokableService*> &)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 6 },
        }}),
        // Signal 'log'
        QtMocHelpers::SignalData<void(const QString &, const QString &)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 8 }, { QMetaType::QString, 9 },
        }}),
        // Signal 'reachedStableState'
        QtMocHelpers::SignalData<void()>(10, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'finished'
        QtMocHelpers::SignalData<void()>(11, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'dataModelChanged'
        QtMocHelpers::SignalData<void(QScxmlDataModel *)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 13, 14 },
        }}),
        // Signal 'initialValuesChanged'
        QtMocHelpers::SignalData<void(const QVariantMap &)>(15, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 16, 17 },
        }}),
        // Signal 'initializedChanged'
        QtMocHelpers::SignalData<void(bool)>(18, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 19 },
        }}),
        // Signal 'loaderChanged'
        QtMocHelpers::SignalData<void(QScxmlCompiler::Loader *)>(20, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 21, 22 },
        }}),
        // Signal 'tableDataChanged'
        QtMocHelpers::SignalData<void(QScxmlTableData *)>(23, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 24, 25 },
        }}),
        // Slot 'start'
        QtMocHelpers::SlotData<void()>(26, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'stop'
        QtMocHelpers::SlotData<void()>(27, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'init'
        QtMocHelpers::SlotData<bool()>(28, 2, QMC::AccessPublic, QMetaType::Bool),
        // Method 'stateNames'
        QtMocHelpers::MethodData<QStringList(bool) const>(29, 2, QMC::AccessPublic, QMetaType::QStringList, {{
            { QMetaType::Bool, 30 },
        }}),
        // Method 'stateNames'
        QtMocHelpers::MethodData<QStringList() const>(29, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::QStringList),
        // Method 'activeStateNames'
        QtMocHelpers::MethodData<QStringList(bool) const>(31, 2, QMC::AccessPublic, QMetaType::QStringList, {{
            { QMetaType::Bool, 30 },
        }}),
        // Method 'activeStateNames'
        QtMocHelpers::MethodData<QStringList() const>(31, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::QStringList),
        // Method 'isActive'
        QtMocHelpers::MethodData<bool(const QString &) const>(32, 2, QMC::AccessPublic, QMetaType::Bool, {{
            { QMetaType::QString, 33 },
        }}),
        // Method 'submitEvent'
        QtMocHelpers::MethodData<void(QScxmlEvent *)>(34, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 35, 36 },
        }}),
        // Method 'submitEvent'
        QtMocHelpers::MethodData<void(const QString &)>(34, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 37 },
        }}),
        // Method 'submitEvent'
        QtMocHelpers::MethodData<void(const QString &, const QVariant &)>(34, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 37 }, { 0x80000000 | 38, 39 },
        }}),
        // Method 'cancelDelayedEvent'
        QtMocHelpers::MethodData<void(const QString &)>(40, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 41 },
        }}),
        // Method 'isDispatchableTarget'
        QtMocHelpers::MethodData<bool(const QString &) const>(42, 2, QMC::AccessPublic, QMetaType::Bool, {{
            { QMetaType::QString, 43 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'running'
        QtMocHelpers::PropertyData<bool>(3, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 0),
        // property 'initialized'
        QtMocHelpers::PropertyData<bool>(19, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Bindable, 7),
        // property 'dataModel'
        QtMocHelpers::PropertyData<QScxmlDataModel*>(44, 0x80000000 | 13, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet | QMC::Bindable, 5),
        // property 'initialValues'
        QtMocHelpers::PropertyData<QVariantMap>(17, 0x80000000 | 16, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet | QMC::Bindable, 6),
        // property 'invokedServices'
        QtMocHelpers::PropertyData<QList<QScxmlInvokableService*>>(6, 0x80000000 | 5, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Bindable, 1),
        // property 'sessionId'
        QtMocHelpers::PropertyData<QString>(45, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'name'
        QtMocHelpers::PropertyData<QString>(46, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'invoked'
        QtMocHelpers::PropertyData<bool>(47, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'parseErrors'
        QtMocHelpers::PropertyData<QList<QScxmlError>>(48, 0x80000000 | 49, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'loader'
        QtMocHelpers::PropertyData<QScxmlCompiler::Loader*>(22, 0x80000000 | 21, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet | QMC::Bindable, 8),
        // property 'tableData'
        QtMocHelpers::PropertyData<QScxmlTableData*>(25, 0x80000000 | 24, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet | QMC::Bindable, 9),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<QScxmlStateMachine, qt_meta_tag_ZN18QScxmlStateMachineE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject QScxmlStateMachine::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18QScxmlStateMachineE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18QScxmlStateMachineE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN18QScxmlStateMachineE_t>.metaTypes,
    nullptr
} };

void QScxmlStateMachine::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QScxmlStateMachine *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->runningChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 1: _t->invokedServicesChanged((*reinterpret_cast<std::add_pointer_t<QList<QScxmlInvokableService*>>>(_a[1]))); break;
        case 2: _t->log((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 3: _t->reachedStableState(); break;
        case 4: _t->finished(); break;
        case 5: _t->dataModelChanged((*reinterpret_cast<std::add_pointer_t<QScxmlDataModel*>>(_a[1]))); break;
        case 6: _t->initialValuesChanged((*reinterpret_cast<std::add_pointer_t<QVariantMap>>(_a[1]))); break;
        case 7: _t->initializedChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 8: _t->loaderChanged((*reinterpret_cast<std::add_pointer_t<QScxmlCompiler::Loader*>>(_a[1]))); break;
        case 9: _t->tableDataChanged((*reinterpret_cast<std::add_pointer_t<QScxmlTableData*>>(_a[1]))); break;
        case 10: _t->start(); break;
        case 11: _t->stop(); break;
        case 12: { bool _r = _t->init();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 13: { QStringList _r = _t->stateNames((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QStringList*>(_a[0]) = std::move(_r); }  break;
        case 14: { QStringList _r = _t->stateNames();
            if (_a[0]) *reinterpret_cast<QStringList*>(_a[0]) = std::move(_r); }  break;
        case 15: { QStringList _r = _t->activeStateNames((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QStringList*>(_a[0]) = std::move(_r); }  break;
        case 16: { QStringList _r = _t->activeStateNames();
            if (_a[0]) *reinterpret_cast<QStringList*>(_a[0]) = std::move(_r); }  break;
        case 17: { bool _r = _t->isActive((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 18: _t->submitEvent((*reinterpret_cast<std::add_pointer_t<QScxmlEvent*>>(_a[1]))); break;
        case 19: _t->submitEvent((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 20: _t->submitEvent((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QVariant>>(_a[2]))); break;
        case 21: _t->cancelDelayedEvent((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 22: { bool _r = _t->isDispatchableTarget((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QList<QScxmlInvokableService*> >(); break;
            }
            break;
        case 5:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QScxmlDataModel* >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (QScxmlStateMachine::*)(bool )>(_a, &QScxmlStateMachine::runningChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (QScxmlStateMachine::*)(const QList<QScxmlInvokableService*> & )>(_a, &QScxmlStateMachine::invokedServicesChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (QScxmlStateMachine::*)(const QString & , const QString & )>(_a, &QScxmlStateMachine::log, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (QScxmlStateMachine::*)()>(_a, &QScxmlStateMachine::reachedStableState, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (QScxmlStateMachine::*)()>(_a, &QScxmlStateMachine::finished, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (QScxmlStateMachine::*)(QScxmlDataModel * )>(_a, &QScxmlStateMachine::dataModelChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (QScxmlStateMachine::*)(const QVariantMap & )>(_a, &QScxmlStateMachine::initialValuesChanged, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (QScxmlStateMachine::*)(bool )>(_a, &QScxmlStateMachine::initializedChanged, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (QScxmlStateMachine::*)(QScxmlCompiler::Loader * )>(_a, &QScxmlStateMachine::loaderChanged, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (QScxmlStateMachine::*)(QScxmlTableData * )>(_a, &QScxmlStateMachine::tableDataChanged, 9))
            return;
    }
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 8:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<QScxmlError> >(); break;
        case 4:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<QScxmlInvokableService*> >(); break;
        case 2:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QScxmlDataModel* >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<bool*>(_v) = _t->isRunning(); break;
        case 1: *reinterpret_cast<bool*>(_v) = _t->isInitialized(); break;
        case 2: *reinterpret_cast<QScxmlDataModel**>(_v) = _t->dataModel(); break;
        case 3: *reinterpret_cast<QVariantMap*>(_v) = _t->initialValues(); break;
        case 4: *reinterpret_cast<QList<QScxmlInvokableService*>*>(_v) = _t->invokedServices(); break;
        case 5: *reinterpret_cast<QString*>(_v) = _t->sessionId(); break;
        case 6: *reinterpret_cast<QString*>(_v) = _t->name(); break;
        case 7: *reinterpret_cast<bool*>(_v) = _t->isInvoked(); break;
        case 8: *reinterpret_cast<QList<QScxmlError>*>(_v) = _t->parseErrors(); break;
        case 9: *reinterpret_cast<QScxmlCompiler::Loader**>(_v) = _t->loader(); break;
        case 10: *reinterpret_cast<QScxmlTableData**>(_v) = _t->tableData(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setRunning(*reinterpret_cast<bool*>(_v)); break;
        case 2: _t->setDataModel(*reinterpret_cast<QScxmlDataModel**>(_v)); break;
        case 3: _t->setInitialValues(*reinterpret_cast<QVariantMap*>(_v)); break;
        case 9: _t->setLoader(*reinterpret_cast<QScxmlCompiler::Loader**>(_v)); break;
        case 10: _t->setTableData(*reinterpret_cast<QScxmlTableData**>(_v)); break;
        default: break;
        }
    }
    if (_c == QMetaObject::BindableProperty) {
        switch (_id) {
        case 1: *static_cast<QUntypedBindable *>(_a[0]) = _t->bindableInitialized(); break;
        case 2: *static_cast<QUntypedBindable *>(_a[0]) = _t->bindableDataModel(); break;
        case 3: *static_cast<QUntypedBindable *>(_a[0]) = _t->bindableInitialValues(); break;
        case 4: *static_cast<QUntypedBindable *>(_a[0]) = _t->bindableInvokedServices(); break;
        case 9: *static_cast<QUntypedBindable *>(_a[0]) = _t->bindableLoader(); break;
        case 10: *static_cast<QUntypedBindable *>(_a[0]) = _t->bindableTableData(); break;
        default: break;
        }
    }
}

const QMetaObject *QScxmlStateMachine::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QScxmlStateMachine::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18QScxmlStateMachineE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int QScxmlStateMachine::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 23)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 23;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 23)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 23;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 11;
    }
    return _id;
}

// SIGNAL 0
void QScxmlStateMachine::runningChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void QScxmlStateMachine::invokedServicesChanged(const QList<QScxmlInvokableService*> & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void QScxmlStateMachine::log(const QString & _t1, const QString & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1, _t2);
}

// SIGNAL 3
void QScxmlStateMachine::reachedStableState()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void QScxmlStateMachine::finished()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void QScxmlStateMachine::dataModelChanged(QScxmlDataModel * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}

// SIGNAL 6
void QScxmlStateMachine::initialValuesChanged(const QVariantMap & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1);
}

// SIGNAL 7
void QScxmlStateMachine::initializedChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1);
}

// SIGNAL 8
void QScxmlStateMachine::loaderChanged(QScxmlCompiler::Loader * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 8, nullptr, _t1);
}

// SIGNAL 9
void QScxmlStateMachine::tableDataChanged(QScxmlTableData * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 9, nullptr, _t1);
}
QT_WARNING_POP
