/****************************************************************************
** Generated QML type registration code
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <QtQml/qqml.h>
#include <QtQml/qqmlmoduleregistration.h>

#if __has_include(<private/eventconnection_p.h>)
#  include <private/eventconnection_p.h>
#endif
#if __has_include(<private/invokedservices_p.h>)
#  include <private/invokedservices_p.h>
#endif
#if __has_include(<private/statemachineextended_p.h>)
#  include <private/statemachineextended_p.h>
#endif
#if __has_include(<private/statemachineloader_p.h>)
#  include <private/statemachineloader_p.h>
#endif


#if !defined(QT_STATIC)
#define Q_QMLTYPE_EXPORT Q_DECL_EXPORT
#else
#define Q_QMLTYPE_EXPORT
#endif
Q_QMLTYPE_EXPORT void qml_register_types_QtScxml()
{
    qmlRegisterModule("QtScxml", 6, 0);
    QT_WARNING_PUSH QT_WARNING_DISABLE_DEPRECATED
    qmlRegisterTypesAndRevisions<QScxmlEventConnection>("QtScxml", 6);
    qmlRegisterTypesAndRevisions<QScxmlEventForeign>("QtScxml", 6);
    QMetaType::fromType<QScxmlEventForeign *>().id();
    qmlRegisterEnum<QScxmlEvent::EventType>("QScxmlEvent::EventType");
    qmlRegisterTypesAndRevisions<QScxmlInvokedServices>("QtScxml", 6);
    QMetaType::fromType<QScxmlStateMachineExtended *>().id();
    qmlRegisterTypesAndRevisions<QScxmlStateMachineForeign>("QtScxml", 6);
    QMetaType::fromType<QScxmlStateMachineForeign *>().id();
    qmlRegisterTypesAndRevisions<QScxmlStateMachineLoader>("QtScxml", 6);
    QT_WARNING_POP
    qmlRegisterModule("QtScxml", 6, 12);
}

static const QQmlModuleRegistration qtScxmlRegistration("QtScxml", qml_register_types_QtScxml);
