/****************************************************************************
** Meta object code from reading C++ file 'qscxmlevent.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../qtscxml/src/scxml/qscxmlevent.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qscxmlevent.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN11QScxmlEventE_t {};
} // unnamed namespace

template <> constexpr inline auto QScxmlEvent::qt_create_metaobjectdata<qt_meta_tag_ZN11QScxmlEventE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QScxmlEvent",
        "clear",
        "",
        "name",
        "eventType",
        "EventType",
        "scxmlType",
        "sendId",
        "origin",
        "originType",
        "invokeId",
        "delay",
        "data",
        "QVariant",
        "errorEvent",
        "errorMessage",
        "PlatformEvent",
        "InternalEvent",
        "ExternalEvent"
    };

    QtMocHelpers::UintData qt_methods {
        // Method 'clear'
        QtMocHelpers::MethodData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'name'
        QtMocHelpers::PropertyData<QString>(3, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'eventType'
        QtMocHelpers::PropertyData<enum EventType>(4, 0x80000000 | 5, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet),
        // property 'scxmlType'
        QtMocHelpers::PropertyData<QString>(6, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'sendId'
        QtMocHelpers::PropertyData<QString>(7, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'origin'
        QtMocHelpers::PropertyData<QString>(8, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'originType'
        QtMocHelpers::PropertyData<QString>(9, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'invokeId'
        QtMocHelpers::PropertyData<QString>(10, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'delay'
        QtMocHelpers::PropertyData<int>(11, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'data'
        QtMocHelpers::PropertyData<QVariant>(12, 0x80000000 | 13, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet),
        // property 'errorEvent'
        QtMocHelpers::PropertyData<bool>(14, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'errorMessage'
        QtMocHelpers::PropertyData<QString>(15, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'EventType'
        QtMocHelpers::EnumData<enum EventType>(5, 5, QMC::EnumFlags{}).add({
            {   16, EventType::PlatformEvent },
            {   17, EventType::InternalEvent },
            {   18, EventType::ExternalEvent },
        }),
    };
    return QtMocHelpers::metaObjectData<QScxmlEvent, qt_meta_tag_ZN11QScxmlEventE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject QScxmlEvent::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11QScxmlEventE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11QScxmlEventE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN11QScxmlEventE_t>.metaTypes,
    nullptr
} };

void QScxmlEvent::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = reinterpret_cast<QScxmlEvent *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->clear(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QString*>(_v) = _t->name(); break;
        case 1: *reinterpret_cast<enum EventType*>(_v) = _t->eventType(); break;
        case 2: *reinterpret_cast<QString*>(_v) = _t->scxmlType(); break;
        case 3: *reinterpret_cast<QString*>(_v) = _t->sendId(); break;
        case 4: *reinterpret_cast<QString*>(_v) = _t->origin(); break;
        case 5: *reinterpret_cast<QString*>(_v) = _t->originType(); break;
        case 6: *reinterpret_cast<QString*>(_v) = _t->invokeId(); break;
        case 7: *reinterpret_cast<int*>(_v) = _t->delay(); break;
        case 8: *reinterpret_cast<QVariant*>(_v) = _t->data(); break;
        case 9: *reinterpret_cast<bool*>(_v) = _t->isErrorEvent(); break;
        case 10: *reinterpret_cast<QString*>(_v) = _t->errorMessage(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setName(*reinterpret_cast<QString*>(_v)); break;
        case 1: _t->setEventType(*reinterpret_cast<enum EventType*>(_v)); break;
        case 3: _t->setSendId(*reinterpret_cast<QString*>(_v)); break;
        case 4: _t->setOrigin(*reinterpret_cast<QString*>(_v)); break;
        case 5: _t->setOriginType(*reinterpret_cast<QString*>(_v)); break;
        case 6: _t->setInvokeId(*reinterpret_cast<QString*>(_v)); break;
        case 7: _t->setDelay(*reinterpret_cast<int*>(_v)); break;
        case 8: _t->setData(*reinterpret_cast<QVariant*>(_v)); break;
        case 10: _t->setErrorMessage(*reinterpret_cast<QString*>(_v)); break;
        default: break;
        }
    }
}
QT_WARNING_POP
