/****************************************************************************
** Meta object code from reading C++ file 'qstatemachine.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../qtscxml/src/statemachine/qstatemachine.h"
#include <QtCore/qmetatype.h>
#include <QtCore/QProperty>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qstatemachine.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN13QStateMachineE_t {};
} // unnamed namespace

template <> constexpr inline auto QStateMachine::qt_create_metaobjectdata<qt_meta_tag_ZN13QStateMachineE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QStateMachine",
        "started",
        "",
        "stopped",
        "runningChanged",
        "running",
        "start",
        "stop",
        "setRunning",
        "_q_start",
        "_q_process",
        "_q_animationFinished",
        "_q_startDelayedEventTimer",
        "_q_killDelayedEventTimer",
        "errorString",
        "globalRestorePolicy",
        "QState::RestorePolicy",
        "animated"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'started'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'stopped'
        QtMocHelpers::SignalData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'runningChanged'
        QtMocHelpers::SignalData<void(bool)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 5 },
        }}),
        // Slot 'start'
        QtMocHelpers::SlotData<void()>(6, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'stop'
        QtMocHelpers::SlotData<void()>(7, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'setRunning'
        QtMocHelpers::SlotData<void(bool)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 5 },
        }}),
        // Slot '_q_start'
        QtMocHelpers::SlotData<void()>(9, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot '_q_process'
        QtMocHelpers::SlotData<void()>(10, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot '_q_animationFinished'
        QtMocHelpers::SlotData<void()>(11, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot '_q_startDelayedEventTimer'
        QtMocHelpers::SlotData<void(int, int)>(12, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 2 }, { QMetaType::Int, 2 },
        }}),
        // Slot '_q_killDelayedEventTimer'
        QtMocHelpers::SlotData<void(int, int)>(13, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 2 }, { QMetaType::Int, 2 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'errorString'
        QtMocHelpers::PropertyData<QString>(14, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Bindable),
        // property 'globalRestorePolicy'
        QtMocHelpers::PropertyData<QState::RestorePolicy>(15, 0x80000000 | 16, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet | QMC::Bindable),
        // property 'running'
        QtMocHelpers::PropertyData<bool>(5, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 2),
        // property 'animated'
        QtMocHelpers::PropertyData<bool>(17, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet | QMC::Bindable),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<QStateMachine, qt_meta_tag_ZN13QStateMachineE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT static const QMetaObject::SuperData qt_meta_extradata_ZN13QStateMachineE[] = {
    QMetaObject::SuperData::link<QState::staticMetaObject>(),
    nullptr
};

Q_CONSTINIT const QMetaObject QStateMachine::staticMetaObject = { {
    QMetaObject::SuperData::link<QState::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13QStateMachineE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13QStateMachineE_t>.data,
    qt_static_metacall,
    qt_meta_extradata_ZN13QStateMachineE,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN13QStateMachineE_t>.metaTypes,
    nullptr
} };

void QStateMachine::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QStateMachine *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->started(QPrivateSignal()); break;
        case 1: _t->stopped(QPrivateSignal()); break;
        case 2: _t->runningChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 3: _t->start(); break;
        case 4: _t->stop(); break;
        case 5: _t->setRunning((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 6: _t->d_func()->_q_start(); break;
        case 7: _t->d_func()->_q_process(); break;
        case 8: _t->d_func()->_q_animationFinished(); break;
        case 9: _t->d_func()->_q_startDelayedEventTimer((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 10: _t->d_func()->_q_killDelayedEventTimer((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (QStateMachine::*)(QPrivateSignal)>(_a, &QStateMachine::started, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (QStateMachine::*)(QPrivateSignal)>(_a, &QStateMachine::stopped, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (QStateMachine::*)(bool )>(_a, &QStateMachine::runningChanged, 2))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QString*>(_v) = _t->errorString(); break;
        case 1: *reinterpret_cast<QState::RestorePolicy*>(_v) = _t->globalRestorePolicy(); break;
        case 2: *reinterpret_cast<bool*>(_v) = _t->isRunning(); break;
        case 3: *reinterpret_cast<bool*>(_v) = _t->isAnimated(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 1: _t->setGlobalRestorePolicy(*reinterpret_cast<QState::RestorePolicy*>(_v)); break;
        case 2: _t->setRunning(*reinterpret_cast<bool*>(_v)); break;
        case 3: _t->setAnimated(*reinterpret_cast<bool*>(_v)); break;
        default: break;
        }
    }
    if (_c == QMetaObject::BindableProperty) {
        switch (_id) {
        case 0: *static_cast<QUntypedBindable *>(_a[0]) = _t->bindableErrorString(); break;
        case 1: *static_cast<QUntypedBindable *>(_a[0]) = _t->bindableGlobalRestorePolicy(); break;
        case 3: *static_cast<QUntypedBindable *>(_a[0]) = _t->bindableAnimated(); break;
        default: break;
        }
    }
}

const QMetaObject *QStateMachine::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QStateMachine::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13QStateMachineE_t>.strings))
        return static_cast<void*>(this);
    return QState::qt_metacast(_clname);
}

int QStateMachine::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QState::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 11)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 11;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 11)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 11;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    }
    return _id;
}

// SIGNAL 0
void QStateMachine::started(QPrivateSignal _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void QStateMachine::stopped(QPrivateSignal _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void QStateMachine::runningChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}
QT_WARNING_POP
