/****************************************************************************
** Generated QML type registration code
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <QtQml/qqml.h>
#include <QtQml/qqmlmoduleregistration.h>

#if __has_include(<private/finalstate_p.h>)
#  include <private/finalstate_p.h>
#endif
#if __has_include(<private/signaltransition_p.h>)
#  include <private/signaltransition_p.h>
#endif
#if __has_include(<private/state_p.h>)
#  include <private/state_p.h>
#endif
#if __has_include(<private/statemachine_p.h>)
#  include <private/statemachine_p.h>
#endif
#if __has_include(<private/statemachineforeign_p.h>)
#  include <private/statemachineforeign_p.h>
#endif
#if __has_include(<private/timeouttransition_p.h>)
#  include <private/timeouttransition_p.h>
#endif


#if !defined(QT_STATIC)
#define Q_QMLTYPE_EXPORT Q_DECL_EXPORT
#else
#define Q_QMLTYPE_EXPORT
#endif
Q_QMLTYPE_EXPORT void qml_register_types_QtQml_StateMachine()
{
    qmlRegisterModule("QtQml.StateMachine", 6, 0);
    QT_WARNING_PUSH QT_WARNING_DISABLE_DEPRECATED
    qmlRegisterTypesAndRevisions<FinalState>("QtQml.StateMachine", 6);
    qmlRegisterTypesAndRevisions<QAbstractStateForeign>("QtQml.StateMachine", 6);
    QMetaType::fromType<QAbstractStateForeign *>().id();
    QMetaType::fromType<QAbstractTransition *>().id();
    qmlRegisterEnum<QAbstractTransition::TransitionType>("QAbstractTransition::TransitionType");
    QMetaType::fromType<QFinalState *>().id();
    qmlRegisterTypesAndRevisions<QHistoryStateForeign>("QtQml.StateMachine", 6);
    QMetaType::fromType<QHistoryStateForeign *>().id();
    qmlRegisterEnum<QHistoryState::HistoryType>("QHistoryState::HistoryType");
    qmlRegisterTypesAndRevisions<QSignalTransitionForeign>("QtQml.StateMachine", 6);
    QMetaType::fromType<QSignalTransitionForeign *>().id();
    qmlRegisterTypesAndRevisions<QStateForeign>("QtQml.StateMachine", 6);
    QMetaType::fromType<QStateForeign *>().id();
    qmlRegisterEnum<QState::ChildMode>("QState::ChildMode");
    qmlRegisterEnum<QState::RestorePolicy>("QState::RestorePolicy");
    QMetaType::fromType<QStateMachine *>().id();
    qmlRegisterTypesAndRevisions<SignalTransition>("QtQml.StateMachine", 6);
    qmlRegisterTypesAndRevisions<State>("QtQml.StateMachine", 6);
    qmlRegisterTypesAndRevisions<StateMachine>("QtQml.StateMachine", 6);
    qmlRegisterTypesAndRevisions<TimeoutTransition>("QtQml.StateMachine", 6);
    QT_WARNING_POP
    qmlRegisterModule("QtQml.StateMachine", 6, 12);
}

static const QQmlModuleRegistration qtQmlStateMachineRegistration("QtQml.StateMachine", qml_register_types_QtQml_StateMachine);
