/****************************************************************************
** Meta object code from reading C++ file 'qscxmlinvokableservice.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../qtscxml/src/scxml/qscxmlinvokableservice.h"
#include "qscxmlstatemachine.h"
#include <QtCore/qmetatype.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qscxmlinvokableservice.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN22QScxmlInvokableServiceE_t {};
} // unnamed namespace

template <> constexpr inline auto QScxmlInvokableService::qt_create_metaobjectdata<qt_meta_tag_ZN22QScxmlInvokableServiceE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QScxmlInvokableService",
        "parentStateMachine",
        "QScxmlStateMachine*",
        "id",
        "name"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
        // property 'parentStateMachine'
        QtMocHelpers::PropertyData<QScxmlStateMachine*>(1, 0x80000000 | 2, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'id'
        QtMocHelpers::PropertyData<QString>(3, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'name'
        QtMocHelpers::PropertyData<QString>(4, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Constant),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<QScxmlInvokableService, qt_meta_tag_ZN22QScxmlInvokableServiceE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject QScxmlInvokableService::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN22QScxmlInvokableServiceE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN22QScxmlInvokableServiceE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN22QScxmlInvokableServiceE_t>.metaTypes,
    nullptr
} };

void QScxmlInvokableService::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QScxmlInvokableService *>(_o);
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QScxmlStateMachine**>(_v) = _t->parentStateMachine(); break;
        case 1: *reinterpret_cast<QString*>(_v) = _t->id(); break;
        case 2: *reinterpret_cast<QString*>(_v) = _t->name(); break;
        default: break;
        }
    }
}

const QMetaObject *QScxmlInvokableService::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QScxmlInvokableService::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN22QScxmlInvokableServiceE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int QScxmlInvokableService::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    }
    return _id;
}
namespace {
struct qt_meta_tag_ZN29QScxmlInvokableServiceFactoryE_t {};
} // unnamed namespace

template <> constexpr inline auto QScxmlInvokableServiceFactory::qt_create_metaobjectdata<qt_meta_tag_ZN29QScxmlInvokableServiceFactoryE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QScxmlInvokableServiceFactory",
        "invokeInfo",
        "QScxmlExecutableContent::InvokeInfo",
        "parameters",
        "QList<QScxmlExecutableContent::ParameterInfo>",
        "names",
        "QList<QScxmlExecutableContent::StringId>"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
        // property 'invokeInfo'
        QtMocHelpers::PropertyData<QScxmlExecutableContent::InvokeInfo>(1, 0x80000000 | 2, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'parameters'
        QtMocHelpers::PropertyData<QList<QScxmlExecutableContent::ParameterInfo>>(3, 0x80000000 | 4, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'names'
        QtMocHelpers::PropertyData<QList<QScxmlExecutableContent::StringId>>(5, 0x80000000 | 6, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<QScxmlInvokableServiceFactory, qt_meta_tag_ZN29QScxmlInvokableServiceFactoryE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject QScxmlInvokableServiceFactory::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN29QScxmlInvokableServiceFactoryE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN29QScxmlInvokableServiceFactoryE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN29QScxmlInvokableServiceFactoryE_t>.metaTypes,
    nullptr
} };

void QScxmlInvokableServiceFactory::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QScxmlInvokableServiceFactory *>(_o);
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QScxmlExecutableContent::InvokeInfo*>(_v) = _t->invokeInfo(); break;
        case 1: *reinterpret_cast<QList<QScxmlExecutableContent::ParameterInfo>*>(_v) = _t->parameters(); break;
        case 2: *reinterpret_cast<QList<QScxmlExecutableContent::StringId>*>(_v) = _t->names(); break;
        default: break;
        }
    }
}

const QMetaObject *QScxmlInvokableServiceFactory::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QScxmlInvokableServiceFactory::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN29QScxmlInvokableServiceFactoryE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int QScxmlInvokableServiceFactory::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    }
    return _id;
}
namespace {
struct qt_meta_tag_ZN31QScxmlStaticScxmlServiceFactoryE_t {};
} // unnamed namespace

template <> constexpr inline auto QScxmlStaticScxmlServiceFactory::qt_create_metaobjectdata<qt_meta_tag_ZN31QScxmlStaticScxmlServiceFactoryE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QScxmlStaticScxmlServiceFactory"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<QScxmlStaticScxmlServiceFactory, qt_meta_tag_ZN31QScxmlStaticScxmlServiceFactoryE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject QScxmlStaticScxmlServiceFactory::staticMetaObject = { {
    QMetaObject::SuperData::link<QScxmlInvokableServiceFactory::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN31QScxmlStaticScxmlServiceFactoryE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN31QScxmlStaticScxmlServiceFactoryE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN31QScxmlStaticScxmlServiceFactoryE_t>.metaTypes,
    nullptr
} };

void QScxmlStaticScxmlServiceFactory::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QScxmlStaticScxmlServiceFactory *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *QScxmlStaticScxmlServiceFactory::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QScxmlStaticScxmlServiceFactory::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN31QScxmlStaticScxmlServiceFactoryE_t>.strings))
        return static_cast<void*>(this);
    return QScxmlInvokableServiceFactory::qt_metacast(_clname);
}

int QScxmlStaticScxmlServiceFactory::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QScxmlInvokableServiceFactory::qt_metacall(_c, _id, _a);
    return _id;
}
namespace {
struct qt_meta_tag_ZN32QScxmlDynamicScxmlServiceFactoryE_t {};
} // unnamed namespace

template <> constexpr inline auto QScxmlDynamicScxmlServiceFactory::qt_create_metaobjectdata<qt_meta_tag_ZN32QScxmlDynamicScxmlServiceFactoryE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QScxmlDynamicScxmlServiceFactory"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<QScxmlDynamicScxmlServiceFactory, qt_meta_tag_ZN32QScxmlDynamicScxmlServiceFactoryE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject QScxmlDynamicScxmlServiceFactory::staticMetaObject = { {
    QMetaObject::SuperData::link<QScxmlInvokableServiceFactory::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN32QScxmlDynamicScxmlServiceFactoryE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN32QScxmlDynamicScxmlServiceFactoryE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN32QScxmlDynamicScxmlServiceFactoryE_t>.metaTypes,
    nullptr
} };

void QScxmlDynamicScxmlServiceFactory::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QScxmlDynamicScxmlServiceFactory *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *QScxmlDynamicScxmlServiceFactory::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QScxmlDynamicScxmlServiceFactory::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN32QScxmlDynamicScxmlServiceFactoryE_t>.strings))
        return static_cast<void*>(this);
    return QScxmlInvokableServiceFactory::qt_metacast(_clname);
}

int QScxmlDynamicScxmlServiceFactory::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QScxmlInvokableServiceFactory::qt_metacall(_c, _id, _a);
    return _id;
}
QT_WARNING_POP
